@@ -7,7 +7,7 @@
  *
  * Gnome-terminal is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * Gnome-terminal is distributed in the hope that it will be useful,
@@ -43,6 +43,9 @@
 
 #ifdef WITH_SMCLIENT
 #include "eggsmclient.h"
+#ifdef GDK_WINDOWING_X11
+#include "eggdesktopfile.h"
+#endif
 #endif
 
 #define FALLBACK_PROFILE_ID "Default"
@@ -109,8 +112,6 @@ struct _TerminalApp
   PangoFontDescription *system_font_desc;
   gboolean enable_mnemonics;
   gboolean enable_menu_accels;
-
-  gboolean use_factory;
 };
 
 enum
@@ -146,6 +147,9 @@ enum
 
 static TerminalApp *global_app = NULL;
 
+/* Evil hack alert: this is exported from libgconf-2 but not in a public header */
+extern gboolean gconf_spawn_daemon(GError** err);
+
 #define MONOSPACE_FONT_DIR "/desktop/gnome/interface"
 #define MONOSPACE_FONT_KEY MONOSPACE_FONT_DIR "/monospace_font_name"
 #define DEFAULT_MONOSPACE_FONT ("Monospace 10")
@@ -686,7 +690,8 @@ profile_list_edit_button_clicked_cb (GtkWidget *button,
   gtk_tree_model_get (model, &iter, (int) COL_PROFILE, &selected_profile, (int) -1);
 
   terminal_app_edit_profile (app, selected_profile,
-                             GTK_WINDOW (app->manage_profiles_dialog));
+                             GTK_WINDOW (app->manage_profiles_dialog),
+                             NULL);
   g_object_unref (selected_profile);
 }
 
@@ -711,7 +716,8 @@ profile_list_row_activated_cb (GtkTreeView       *tree_view,
   gtk_tree_model_get (model, &iter, (int) COL_PROFILE, &selected_profile, (int) -1);
 
   terminal_app_edit_profile (app, selected_profile,
-                             GTK_WINDOW (app->manage_profiles_dialog));
+                             GTK_WINDOW (app->manage_profiles_dialog),
+                             NULL);
   g_object_unref (selected_profile);
 }
 
@@ -918,18 +924,17 @@ terminal_app_encoding_list_notify_cb (GConfClient *client,
   /* Mark all as non-active, then re-enable the active ones */
   g_hash_table_foreach (app->encodings, (GHFunc) encoding_mark_active, GUINT_TO_POINTER (FALSE));
 
-  /* First add the local encoding. */
-  if (!g_get_charset (&charset))
-    {
-      encoding = g_hash_table_lookup (app->encodings, charset);
-      if (encoding)
-        encoding->is_active = TRUE;
-    }
+  /* First add the locale's charset */
+  encoding = g_hash_table_lookup (app->encodings, "current");
+  g_assert (encoding);
+  if (terminal_encoding_is_valid (encoding))
+    encoding->is_active = TRUE;
 
-  /* Always ensure that UTF-8 is available. */
+  /* Also always make UTF-8 available */
   encoding = g_hash_table_lookup (app->encodings, "UTF-8");
   g_assert (encoding);
-  encoding->is_active = TRUE;
+  if (terminal_encoding_is_valid (encoding))
+    encoding->is_active = TRUE;
 
   val = gconf_entry_get_value (entry);
   if (val != NULL &&
@@ -947,20 +952,7 @@ terminal_app_encoding_list_notify_cb (GConfClient *client,
       if (!charset)
         continue;
 
-      /* We already handled the locale charset above */
-      if (strcmp (charset, "current") == 0)
-        continue; 
-
-      encoding = g_hash_table_lookup (app->encodings, charset);
-      if (!encoding)
-        {
-          encoding = terminal_encoding_new (charset,
-                                            _("User Defined"),
-                                            TRUE,
-                                            TRUE /* scary! */);
-          g_hash_table_insert (app->encodings, encoding->charset, encoding);
-        }
-
+      encoding = terminal_app_ensure_encoding (app, charset);
       if (!terminal_encoding_is_valid (encoding))
         continue;
 
@@ -1139,7 +1131,7 @@ new_profile_response_cb (GtkWidget *new_profile_dialog,
                              list,
                              NULL);
 
-      terminal_profile_edit (new_profile, transient_parent);
+      terminal_profile_edit (new_profile, transient_parent, NULL);
 
     cleanup:
       g_free (name);
@@ -1378,8 +1370,26 @@ G_DEFINE_TYPE (TerminalApp, terminal_app, G_TYPE_OBJECT)
 static void
 terminal_app_init (TerminalApp *app)
 {
+  GError *error = NULL;
+
   global_app = app;
 
+  /* If the gconf daemon isn't available (e.g. because there's no dbus
+   * session bus running), we'd crash later on. Tell the user about it
+   * now, and exit. See bug #561663.
+   * Don't use gconf_ping_daemon() here since the server may just not
+   * be running yet, but able to be started. See comments on bug #564649.
+   */
+  if (!gconf_spawn_daemon (&error))
+    {
+      g_printerr ("Failed to summon the GConf demon; exiting.  %s\n", error->message);
+      g_error_free (error);
+
+      exit (EXIT_FAILURE);
+    }
+
+  gtk_window_set_default_icon_name (GNOME_TERMINAL_ICON_NAME);
+
   /* Initialise defaults */
   app->enable_mnemonics = DEFAULT_ENABLE_MNEMONICS;
   app->enable_menu_accels = DEFAULT_ENABLE_MENU_BAR_ACCEL;
@@ -1396,6 +1406,12 @@ terminal_app_init (TerminalApp *app)
   gconf_client_add_dir (app->conf, MONOSPACE_FONT_DIR,
                         GCONF_CLIENT_PRELOAD_ONELEVEL,
                         NULL);
+  gconf_client_add_dir (app->conf, CONF_PROXY_PREFIX,
+                        GCONF_CLIENT_PRELOAD_ONELEVEL,
+                        NULL);
+  gconf_client_add_dir (app->conf, CONF_HTTP_PROXY_PREFIX,
+                        GCONF_CLIENT_PRELOAD_ONELEVEL,
+                        NULL);
 
   app->profile_list_notify_id =
     gconf_client_notify_add (app->conf, PROFILE_LIST_KEY,
@@ -1449,6 +1465,16 @@ terminal_app_init (TerminalApp *app)
 #ifdef WITH_SMCLIENT
 {
   EggSMClient *sm_client;
+#ifdef GDK_WINDOWING_X11
+  char *desktop_file;
+
+  desktop_file = g_build_filename (TERM_DATADIR,
+                                   "applications",
+                                   PACKAGE ".desktop",
+                                   NULL);
+  egg_set_desktop_file_without_defaults (desktop_file);
+  g_free (desktop_file);
+#endif /* GDK_WINDOWING_X11 */
 
   sm_client = egg_sm_client_get ();
   g_signal_connect (sm_client, "save-state",
@@ -1563,6 +1589,12 @@ terminal_app_set_property (GObject *object,
     }
 }
 
+static void
+terminal_app_real_quit (TerminalApp *app)
+{
+  gtk_main_quit();
+}
+
 static void
 terminal_app_class_init (TerminalAppClass *klass)
 {
@@ -1572,6 +1604,8 @@ terminal_app_class_init (TerminalAppClass *klass)
   object_class->get_property = terminal_app_get_property;
   object_class->set_property = terminal_app_set_property;
 
+  klass->quit = terminal_app_real_quit;
+
   signals[QUIT] =
     g_signal_new (I_("quit"),
                   G_OBJECT_CLASS_TYPE (object_class),
@@ -1630,31 +1664,27 @@ terminal_app_class_init (TerminalAppClass *klass)
 
 /* Public API */
 
-void
-terminal_app_initialize (gboolean use_factory)
+TerminalApp*
+terminal_app_get (void)
 {
-  g_assert (global_app == NULL);
-  g_object_new (TERMINAL_TYPE_APP, NULL);
-  g_assert (global_app != NULL);
+  if (global_app == NULL) {
+    g_object_new (TERMINAL_TYPE_APP, NULL);
+    g_assert (global_app != NULL);
+  }
 
-  global_app->use_factory = use_factory;
+  return global_app;
 }
 
 void
 terminal_app_shutdown (void)
 {
-  g_assert (global_app != NULL);
+  if (global_app == NULL)
+    return;
+
   g_object_unref (global_app);
   g_assert (global_app == NULL);
 }
 
-TerminalApp*
-terminal_app_get (void)
-{
-  g_assert (global_app != NULL);
-  return global_app;
-}
-
 /**
  * terminal_app_handle_options:
  * @app:
@@ -1728,6 +1758,11 @@ terminal_app_handle_options (TerminalApp *app,
   /* Make sure we open at least one window */
   terminal_options_ensure_window (options);
 
+  if (options->startup_id != NULL)
+    _terminal_debug_print (TERMINAL_DEBUG_FACTORY,
+                           "Startup ID is '%s'\n",
+                           options->startup_id);
+
   for (lw = options->initial_windows;  lw != NULL; lw = lw->next)
     {
       InitialWindow *iw = lw->data;
@@ -1743,8 +1778,8 @@ terminal_app_handle_options (TerminalApp *app,
       if (iw->source_tag == SOURCE_SESSION)
         terminal_window_set_is_restored (window);
 
-      if (options->startup_id)
-        terminal_window_set_startup_id (window, options->startup_id);
+      if (options->startup_id != NULL)
+        gtk_window_set_startup_id (GTK_WINDOW (window), options->startup_id);
 
       /* Overwrite the default, unique window role set in terminal_window_init */
       if (iw->role)
@@ -1859,17 +1894,16 @@ terminal_app_new_terminal (TerminalApp     *app,
   terminal_window_switch_screen (window, screen);
   gtk_widget_grab_focus (GTK_WIDGET (screen));
 
-  terminal_screen_launch_child (screen);
-
   return screen;
 }
 
 void
 terminal_app_edit_profile (TerminalApp     *app,
                            TerminalProfile *profile,
-                           GtkWindow       *transient_parent)
+                           GtkWindow       *transient_parent,
+                           const char      *widget_name)
 {
-  terminal_profile_edit (profile, transient_parent);
+  terminal_profile_edit (profile, transient_parent, widget_name);
 }
 
 void
@@ -1972,6 +2006,34 @@ terminal_app_get_encodings (TerminalApp *app)
   return app->encodings;
 }
 
+/**
+ * terminal_app_ensure_encoding:
+ * @app:
+ * @charset:
+ *
+ * Ensures there's a #TerminalEncoding for @charset available.
+ */
+TerminalEncoding *
+terminal_app_ensure_encoding (TerminalApp *app,
+                              const char *charset)
+{
+  TerminalEncoding *encoding;
+
+  encoding = g_hash_table_lookup (app->encodings, charset);
+  if (encoding == NULL)
+    {
+      encoding = terminal_encoding_new (charset,
+                                        _("User Defined"),
+                                        TRUE,
+                                        TRUE /* scary! */);
+      g_hash_table_insert (app->encodings,
+                          (gpointer) terminal_encoding_get_id (encoding),
+                          encoding);
+    }
+
+  return encoding;
+}
+
 /**
  * terminal_app_get_active_encodings:
  *
@@ -2013,9 +2075,6 @@ terminal_app_save_config (TerminalApp *app,
   g_key_file_set_integer (key_file, TERMINAL_CONFIG_GROUP, TERMINAL_CONFIG_PROP_VERSION, TERMINAL_CONFIG_VERSION);
   g_key_file_set_integer (key_file, TERMINAL_CONFIG_GROUP, TERMINAL_CONFIG_PROP_COMPAT_VERSION, TERMINAL_CONFIG_COMPAT_VERSION);
 
-  /* FIXMEchpe this seems useless */
-  g_key_file_set_boolean (key_file, TERMINAL_CONFIG_GROUP, TERMINAL_CONFIG_PROP_FACTORY, app->use_factory);
-
   window_names_array = g_ptr_array_sized_new (g_list_length (app->windows) + 1);
 
   for (lw = app->windows; lw != NULL; lw = lw->next)
